@@ -8,9 +8,11 @@ import re
 import sys
 import unittest
 import uuid
+import warnings
 import yaml
 from calendar import Calendar
 from contextlib import redirect_stderr, redirect_stdout
+from copy import deepcopy
 from datetime import datetime
 from enum import Enum
 from gzip import GzipFile
@@ -226,6 +228,51 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual(parser.dump(cfg), 'complex: (2+3j)\n')
 
 
+    def test_list_append(self):
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--val', type=Union[int, str, List[int]])
+        self.assertEqual(0, parser.parse_args(['--val+=0']).val)
+        self.assertEqual([1, 2, 3], parser.parse_args(['--val+=1', '--val+=2', '--val+=3']).val)
+        self.assertEqual([1, 2, 3], parser.parse_args(['--val=[1,2]', '--val+=3']).val)
+        self.assertEqual(1, parser.parse_args(['--val=a', '--val+=1']).val)
+        with warnings.catch_warnings(record=True) as w:
+            self.assertEqual(3, parser.parse_args(['--val=[1,2]', '--val=3']).val)
+            self.assertIn('Replacing list value "[1, 2]" with "3"', str(w[0].message))
+
+
+    def test_list_append_config(self):
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--cfg', action=ActionConfigFile)
+        parser.add_argument('--val', type=List[int], default=[1, 2])
+        self.assertEqual([3, 4], parser.parse_args(['--cfg', 'val: [3, 4]']).val)
+        self.assertEqual([1, 2, 3], parser.parse_args(['--cfg', 'val+: 3']).val)
+        self.assertEqual([1, 2, 3, 4], parser.parse_args(['--cfg', 'val+: [3, 4]']).val)
+        self.assertRaises(ParserError, lambda: parser.parse_args(['--cfg', 'val+: a']))
+
+
+    def test_list_append_subcommand_subclass(self):
+        class A:
+            def __init__(self, cals: Union[Calendar, List[Calendar]] = None):
+                self.cals = cals
+
+        parser = ArgumentParser(error_handler=None)
+        subparser = ArgumentParser()
+        subparser.add_class_arguments(A, 'a')
+        subcommands = parser.add_subcommands()
+        subcommands.add_subcommand('cmd', subparser)
+        cfg = parser.parse_args([
+            'cmd',
+            '--a.cals+=Calendar',
+            '--a.cals.firstweekday=3',
+            '--a.cals+=TextCalendar',
+            '--a.cals.firstweekday=1',
+        ])
+        self.assertEqual(['calendar.Calendar', 'calendar.TextCalendar'], [x.class_path for x in cfg.cmd.a.cals])
+        self.assertEqual([3, 1], [x.init_args.firstweekday for x in cfg.cmd.a.cals])
+        cfg = parser.parse_args(['cmd', f'--a={json.dumps(cfg.cmd.a.as_dict())}', '--a.cals.firstweekday=4'])
+        self.assertEqual(Namespace(firstweekday=4), cfg.cmd.a.cals[-1].init_args)
+
+
     def test_restricted_number_type(self):
         limit_val = random.randint(100, 10000)
         larger_than = restricted_number_type(f'larger_than_{limit_val}', int, ('>', limit_val))
@@ -324,6 +371,15 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual('uuid: '+str(id1)+'\nuuids:\n- '+str(id1)+'\n- '+str(id2)+'\n', parser.dump(cfg))
 
 
+    @unittest.skipIf(sys.version_info[:2] < (3, 10), 'new union syntax introduced in python 3.10')
+    def test_union_new_syntax(self):
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--str', type=eval('int | None'))
+        self.assertEqual(123, parser.parse_args(['--str=123']).str)
+        self.assertIsNone(parser.parse_args(['--str=null']).str)
+        self.assertRaises(ParserError, lambda: parser.parse_args(['--str=abc']))
+
+
     def test_Callable_with_function_path(self):
         parser = ArgumentParser(error_handler=None)
         parser.add_argument('--callable', type=Callable, default=lazy_instance)
@@ -504,6 +560,37 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual(cfg.op.init_args, Namespace(firstweekday=2))
 
 
+    def test_class_type_config_merge_init_args(self):
+        class MyCal(Calendar):
+            def __init__(self, param_a: int = 1, param_b: str = 'x', **kwargs):
+                super().__init__(**kwargs)
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--cfg', action=ActionConfigFile)
+        parser.add_argument('--cal', type=Calendar)
+
+        with mock_module(MyCal) as module:
+            config1 = {
+                'cal': {
+                    'class_path': f'{module}.MyCal',
+                    'init_args': {
+                        'firstweekday': 2,
+                        'param_b': 'y',
+                    }
+                }
+            }
+            config2 = deepcopy(config1)
+            config2['cal']['init_args'] = {
+                'param_a': 2,
+                'firstweekday': 3,
+            }
+            expected = deepcopy(config1['cal'])
+            expected['init_args'].update(config2['cal']['init_args'])
+
+            cfg = parser.parse_args([f'--cfg={yaml.safe_dump(config1)}', f'--cfg={yaml.safe_dump(config2)}'])
+            self.assertEqual(cfg.cal.as_dict(), expected)
+
+
     def test_class_type_subclass_nested_init_args(self):
         class Class:
             def __init__(self, cal: Calendar, p1: int = 0):
